@@ -50,12 +50,13 @@ class SyncJournalFileRecord;
 
 namespace EncryptionStatusEnums {
 
-Q_NAMESPACE
+OCSYNC_EXPORT Q_NAMESPACE
 
 enum class ItemEncryptionStatus : int {
     NotEncrypted = 0,
     Encrypted = 1,
     EncryptedMigratedV1_2 = 2,
+    EncryptedMigratedV2_0 = 3,
 };
 
 Q_ENUM_NS(ItemEncryptionStatus)
@@ -65,6 +66,7 @@ enum class JournalDbEncryptionStatus : int {
     Encrypted = 1,
     EncryptedMigratedV1_2Invalid = 2,
     EncryptedMigratedV1_2 = 3,
+    EncryptedMigratedV2_0 = 4,
 };
 
 Q_ENUM_NS(JournalDbEncryptionStatus)
@@ -73,6 +75,8 @@ ItemEncryptionStatus fromDbEncryptionStatus(JournalDbEncryptionStatus encryption
 
 JournalDbEncryptionStatus toDbEncryptionStatus(ItemEncryptionStatus encryptionStatus);
 
+ItemEncryptionStatus fromEndToEndEncryptionApiVersion(const double version);
+
 }
 
 }
@@ -149,6 +153,7 @@ enum SyncInstructions {
     CSYNC_INSTRUCTION_UPDATE_METADATA     = 1 << 10, /* If the etag has been updated and need to be writen to the db,
                                                         but without any propagation (UPDATE|RECONCILE) */
     CSYNC_INSTRUCTION_CASE_CLASH_CONFLICT = 1 << 12, /* The file need to be downloaded because it is a case clash conflict (RECONCILE) */
+    CSYNC_INSTRUCTION_UPDATE_VFS_METADATA = 1 << 13, /* vfs item metadata are out of sync and we need to tell operating system about it */
 };
 
 Q_ENUM_NS(SyncInstructions)
