@@ -27,6 +27,7 @@
 #include "networkjobs.h"
 #include "clientproxy.h"
 #include <creds/abstractcredentials.h>
+#include "systray.h"
 
 namespace OCC {
 
@@ -36,8 +37,9 @@ Q_LOGGING_CATEGORY(lcConnectionValidator, "nextcloud.sync.connectionvalidator",
 // This makes sure we get tried often enough without "ConnectionValidator already running"
 static qint64 timeoutToUseMsec = qMax(1000, ConnectionValidator::DefaultCallingIntervalMsec - 5 * 1000);
 
-ConnectionValidator::ConnectionValidator(AccountStatePtr accountState, QObject *parent)
+ConnectionValidator::ConnectionValidator(AccountStatePtr accountState, const QStringList &previousErrors, QObject *parent)
     : QObject(parent)
+    , _previousErrors(previousErrors)
     , _accountState(accountState)
     , _account(accountState->account())
 {
@@ -328,7 +330,20 @@ void ConnectionValidator::reportConnected() {
 void ConnectionValidator::reportResult(Status status)
 {
     emit connectionResult(status, _errors);
+
+    // notify user of errors
+    if (!_errors.isEmpty() && _previousErrors != _errors) {
+       showSystrayErrorMessage();
+    }
+
     deleteLater();
 }
 
+void ConnectionValidator::showSystrayErrorMessage()
+{
+    Systray::instance()->showMessage(tr("Connection issue"),
+                                     _errors.join("<br>"),
+                                     QSystemTrayIcon::Warning);
+}
+
 } // namespace OCC
