@@ -206,14 +206,15 @@ void EditLocallyJob::fetchRemoteFileParentInfo()
         return;
     }
 
-    const auto job = new LsColJob(_accountState->account(), QDir::cleanPath(_folderForFile->remotePathTrailingSlash() + _relPathParent), this);
+    const auto job = new LsColJob(_accountState->account(), QDir::cleanPath(_folderForFile->remotePathTrailingSlash() + _relPathParent));
     const QList<QByteArray> props{QByteArrayLiteral("resourcetype"),
                                   QByteArrayLiteral("getlastmodified"),
                                   QByteArrayLiteral("getetag"),
                                   QByteArrayLiteral("http://owncloud.org/ns:size"),
                                   QByteArrayLiteral("http://owncloud.org/ns:id"),
                                   QByteArrayLiteral("http://owncloud.org/ns:permissions"),
-                                  QByteArrayLiteral("http://owncloud.org/ns:checksums")};
+                                  QByteArrayLiteral("http://owncloud.org/ns:checksums"),
+                                  QByteArrayLiteral("http://nextcloud.org/ns:is-mount-root")};
 
     job->setProperties(props);
     connect(job, &LsColJob::directoryListingIterated, this, &EditLocallyJob::slotDirectoryListingIterated);
@@ -545,7 +546,9 @@ void EditLocallyJob::slotDirectoryListingIterated(const QString &name, const QMa
         const auto cleanName = nameWithoutDavPath.startsWith(remoteFolderPathWithoutLeadingSlash)
             ? nameWithoutDavPath.mid(remoteFolderPathWithoutLeadingSlash.size()) : nameWithoutDavPath;
         disconnect(job, &LsColJob::directoryListingIterated, this, &EditLocallyJob::slotDirectoryListingIterated);
-        _fileParentItem = SyncFileItem::fromProperties(cleanName, properties);
+        _fileParentItem = SyncFileItem::fromProperties(cleanName,
+                                                       properties,
+                                                       _accountState->account()->serverHasMountRootProperty() ? RemotePermissions::MountedPermissionAlgorithm::UseMountRootProperty : RemotePermissions::MountedPermissionAlgorithm::WildGuessMountedSubProperty);
     }
 }
 
@@ -650,7 +653,8 @@ void EditLocallyJob::lockFile()
                                                                 _folderForFile->remotePathTrailingSlash(),
                                                                 _folderForFile->path(),
                                                                 _folderForFile->journalDb(),
-                                                                SyncFileItem::LockStatus::LockedItem);
+                                                                SyncFileItem::LockStatus::LockedItem,
+                                                                SyncFileItem::LockOwnerType::TokenLock);
 }
 
 void EditLocallyJob::disconnectFolderSignals()
