@@ -18,6 +18,7 @@
 
 #include "fileproviderdomainmanager.h"
 #include "fileprovidersocketserver.h"
+#include "fileproviderxpc.h"
 
 namespace OCC {
 
@@ -36,11 +37,17 @@ public:
     static FileProvider *instance();
     ~FileProvider() override;
 
-    static bool fileProviderAvailable();
+    [[nodiscard]] static bool fileProviderAvailable();
+
+    [[nodiscard]] FileProviderXPC *xpc() const;
+
+private slots:
+    void configureXPC();
 
 private:
     std::unique_ptr<FileProviderDomainManager> _domainManager;
     std::unique_ptr<FileProviderSocketServer> _socketServer;
+    std::unique_ptr<FileProviderXPC> _xpc;
 
     static FileProvider *_instance;
     explicit FileProvider(QObject * const parent = nullptr);
