@@ -17,6 +17,8 @@
 #include "account.h"
 #include "folderman.h"
 #include "accountstate.h"
+#include "clientsideencryption.h"
+#include "updatee2eefolderusersmetadatajob.h"
 
 #include <QUrl>
 #include <QJsonDocument>
@@ -486,6 +488,40 @@ void ShareManager::createShare(const QString &path,
     job->getSharedWithMe();
 }
 
+void ShareManager::createE2EeShareJob(const QString &fullRemotePath,
+                                      const ShareePtr sharee,
+                                      const Share::Permissions permissions,
+                                      const QString &password)
+{
+    Folder *folder = nullptr;
+    for (const auto &f : FolderMan::instance()->map()) {
+        if (f->accountState()->account() != _account) {
+            continue;
+        }
+        folder = f;
+    }
+
+    if (!folder) {
+        emit serverError(0, "Failed creating share");
+        return;
+    }
+
+    Q_ASSERT(folder->remotePath() == QStringLiteral("/") ||
+        Utility::noLeadingSlashPath(fullRemotePath).startsWith(Utility::noLeadingSlashPath(Utility::noTrailingSlashPath(folder->remotePath()))));
+
+    const auto createE2eeShareJob = new UpdateE2eeFolderUsersMetadataJob(_account,
+                                                                         folder->journalDb(),
+                                                                         folder->remotePath(),
+                                                                         UpdateE2eeFolderUsersMetadataJob::Add,
+                                                                         fullRemotePath,
+                                                                         sharee->shareWith(),
+                                                                         QSslCertificate{},
+                                                                         this);
+
+    createE2eeShareJob->setUserData({sharee, permissions, password});
+    connect(createE2eeShareJob, &UpdateE2eeFolderUsersMetadataJob::finished, this, &ShareManager::slotCreateE2eeShareJobFinised);
+    createE2eeShareJob->start();
+}
 
 void ShareManager::slotShareCreated(const QJsonDocument &reply)
 {
@@ -630,4 +666,28 @@ void ShareManager::slotOcsError(int statusCode, const QString &message)
 {
     emit serverError(statusCode, message);
 }
+
+
+void ShareManager::slotCreateE2eeShareJobFinised(int statusCode, const QString &message)
+{
+    const auto job = qobject_cast<UpdateE2eeFolderUsersMetadataJob *>(sender());
+    Q_ASSERT(job);
+    if (!job) {
+        qCWarning(lcUserGroupShare) << "slotCreateE2eeShareJobFinised must be called by UpdateE2eeShareMetadataJob::finished signal!";
+        return;
+    }
+    disconnect(job, &UpdateE2eeFolderUsersMetadataJob::finished, this, &ShareManager::slotCreateE2eeShareJobFinised);
+    const auto userData = job->userData();
+    Q_ASSERT(userData.sharee);
+    if (!userData.sharee) {
+        qCWarning(lcUserGroupShare) << "missing userData Map in UpdateE2eeShareMetadataJob instance!";
+        emit serverError(-1, tr("Error"));
+        return;
+    }
+    if (statusCode != 200) {
+        emit serverError(statusCode, message);
+    } else {
+        createShare(job->path(), Share::ShareType(userData.sharee->type()), userData.sharee->shareWith(), userData.desiredPermissions, userData.password);
+    }
+}
 }
