@@ -688,7 +688,9 @@ void ActivityListModel::checkAndRemoveSeenActivities(const OCC::ActivityList &ne
 {
     ActivityList activitiesToRemove;
     for (const auto &activity : _finalList) {
-        if (activity._objectType == QStringLiteral("chat") && !newActivities.contains(activity)) {
+        const auto isTalkActiity = activity._objectType == QStringLiteral("chat") ||
+            activity._objectType == QStringLiteral("call");
+        if (isTalkActiity && !newActivities.contains(activity)) {
             activitiesToRemove.push_back(activity);
         }
     }
@@ -883,7 +885,9 @@ QVariantList ActivityListModel::convertLinksToActionButtons(const Activity &acti
 
         // Use the isDismissable model role to present custom dismiss button if needed
         // Also don't show "View chat" for talk activities, default action will open chat anyway
-        if (activityLink._verb == "DELETE" || (activityLink._verb == "WEB" && activity._objectType == "chat")) {
+        const auto isUseCustomDeleteAction = activityLink._verb == "DELETE"
+            && activity._objectType != QStringLiteral("remote_share");
+        if (isUseCustomDeleteAction || (activityLink._verb == "WEB" && activity._objectType == "chat")) {
             continue;
         }
 
