@@ -119,12 +119,12 @@ bool User::canShowNotification(const long notificationId)
             !notificationAlreadyShown(notificationId);
 }
 
-void User::checkAndRemoveSeenActivities(const ActivityList &list, const int numChatNotificationsReceived)
+void User::checkAndRemoveSeenActivities(const ActivityList &list, const int numTalkNotificationsReceived)
 {
-    if (numChatNotificationsReceived < _lastChatNotificationsReceivedCount) {
+    if (numTalkNotificationsReceived < _lastTalkNotificationsReceivedCount) {
         _activityModel->checkAndRemoveSeenActivities(list);
     }
-    _lastChatNotificationsReceivedCount = numChatNotificationsReceived;
+    _lastTalkNotificationsReceivedCount = numTalkNotificationsReceived;
 }
 
 void User::showDesktopNotification(const QString &title, const QString &message, const long notificationId)
@@ -205,10 +205,11 @@ void User::showDesktopTalkNotification(const Activity &activity)
 
 void User::slotBuildNotificationDisplay(const ActivityList &list)
 {
-    const auto chatNotificationsReceivedCount = std::count_if(std::cbegin(list), std::cend(list), [](const auto &activity) {
-        return activity._objectType == QStringLiteral("chat");
+    const auto talkNotificationsReceivedCount = std::count_if(std::cbegin(list), std::cend(list), [](const auto &activity) {
+        return activity._objectType == QStringLiteral("chat") ||
+            activity._objectType == QStringLiteral("call");
     });
-    checkAndRemoveSeenActivities(list, chatNotificationsReceivedCount);
+    checkAndRemoveSeenActivities(list, talkNotificationsReceivedCount);
 
     ActivityList toNotifyList;
 
