@@ -32,6 +32,9 @@
 #include <qstack.h>
 #include <QCoreApplication>
 
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+#include <filesystem>
+#endif
 #include <ctime>
 
 
@@ -59,6 +62,9 @@ bool PropagateLocalRemove::removeRecursively(const QString &path)
     QString absolute = propagator()->fullLocalPath(_item->_file + path);
     QStringList errors;
     QList<QPair<QString, bool>> deleted;
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+    FileSystem::setFolderPermissions(absolute, FileSystem::FolderPermissions::ReadWrite);
+#endif
     bool success = FileSystem::removeRecursively(
         absolute,
         [&deleted](const QString &path, bool isDir) {
@@ -129,7 +135,7 @@ void PropagateLocalRemove::start()
     }
     propagator()->reportProgress(*_item, 0);
     if (!propagator()->_journal->deleteFileRecord(_item->_originalFile, _item->isDirectory())) {
-        qCWarning(lcPropagateLocalRename) << "could not delete file from local DB" << _item->_originalFile;
+        qCWarning(lcPropagateLocalRemove()) << "could not delete file from local DB" << _item->_originalFile;
         done(SyncFileItem::NormalError, tr("Could not delete file record %1 from local DB").arg(_item->_originalFile), ErrorCategory::GenericError);
         return;
     }
@@ -181,6 +187,26 @@ void PropagateLocalMkdir::startLocalMkdir()
         done(SyncFileItem::FileNameClash, tr("Folder %1 cannot be created because of a local file or folder name clash!").arg(newDirStr), ErrorCategory::GenericError);
         return;
     }
+
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+    auto parentFolderPath = std::filesystem::path{};
+    auto parentNeedRollbackPermissions = false;
+    try {
+        const auto newDirPath = std::filesystem::path{newDirStr.toStdWString()};
+        Q_ASSERT(newDirPath.has_parent_path());
+        parentFolderPath = newDirPath.parent_path();
+        if (FileSystem::isFolderReadOnly(parentFolderPath)) {
+            FileSystem::setFolderPermissions(QString::fromStdWString(parentFolderPath.wstring()), FileSystem::FolderPermissions::ReadWrite);
+            parentNeedRollbackPermissions = true;
+            emit propagator()->touchedFile(QString::fromStdWString(parentFolderPath.wstring()));
+        }
+    }
+    catch (const std::filesystem::filesystem_error &e)
+    {
+        qCWarning(lcPropagateLocalMkdir) << "exception when checking parent folder access rights" << e.what() << e.path1().c_str() << e.path2().c_str();
+    }
+#endif
+
     emit propagator()->touchedFile(newDirStr);
     QDir localDir(propagator()->localPath());
     if (!localDir.mkpath(_item->_file)) {
@@ -188,6 +214,35 @@ void PropagateLocalMkdir::startLocalMkdir()
         return;
     }
 
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+    if (!_item->_remotePerm.isNull() &&
+        !_item->_remotePerm.hasPermission(RemotePermissions::CanAddFile) &&
+        !_item->_remotePerm.hasPermission(RemotePermissions::CanRename) &&
+        !_item->_remotePerm.hasPermission(RemotePermissions::CanMove) &&
+        !_item->_remotePerm.hasPermission(RemotePermissions::CanAddSubDirectories)) {
+        try {
+            FileSystem::setFolderPermissions(newDirStr, FileSystem::FolderPermissions::ReadOnly);
+        }
+        catch (const std::filesystem::filesystem_error &e)
+        {
+            qCWarning(lcPropagateLocalMkdir) << "exception when checking parent folder access rights" << e.what() << e.path1().c_str() << e.path2().c_str();
+            done(SyncFileItem::NormalError, tr("The folder %1 cannot be made read-only: %2").arg(_item->_file, e.what()), ErrorCategory::GenericError);
+            return;
+        }
+    }
+
+    try {
+        if (parentNeedRollbackPermissions) {
+            FileSystem::setFolderPermissions(QString::fromStdWString(parentFolderPath.wstring()), FileSystem::FolderPermissions::ReadOnly);
+            emit propagator()->touchedFile(QString::fromStdWString(parentFolderPath.wstring()));
+        }
+    }
+    catch (const std::filesystem::filesystem_error &e)
+    {
+        qCWarning(lcPropagateLocalMkdir) << "exception when checking parent folder access rights" << e.what() << e.path1().c_str() << e.path2().c_str();
+    }
+#endif
+
     // Insert the directory into the database. The correct etag will be set later,
     // once all contents have been propagated, because should_update_metadata is true.
     // Adding an entry with a dummy etag to the database still makes sense here
@@ -224,10 +279,10 @@ void PropagateLocalRename::start()
 
     auto &vfs = propagator()->syncOptions()._vfs;
     const auto previousNameInDb = propagator()->adjustRenamedPath(_item->_file);
-    const auto existingFile = propagator()->fullLocalPath(propagator()->adjustRenamedPath(_item->_file));
+    const auto existingFile = propagator()->fullLocalPath(previousNameInDb);
     const auto targetFile = propagator()->fullLocalPath(_item->_renameTarget);
 
-    const auto fileAlreadyMoved = !QFileInfo::exists(propagator()->fullLocalPath(_item->_originalFile));
+    const auto fileAlreadyMoved = !QFileInfo::exists(propagator()->fullLocalPath(_item->_originalFile)) && QFileInfo::exists(existingFile);
     auto pinState = OCC::PinState::Unspecified;
     if (!fileAlreadyMoved) {
         auto pinStateResult = vfs->pinState(propagator()->adjustRenamedPath(_item->_file));
@@ -239,6 +294,7 @@ void PropagateLocalRename::start()
     // if the file is a file underneath a moved dir, the _item->file is equal
     // to _item->renameTarget and the file is not moved as a result.
     qCDebug(lcPropagateLocalRename) << _item->_file << _item->_renameTarget << _item->_originalFile << previousNameInDb << (fileAlreadyMoved ? "original file has already moved" : "original file is still there");
+    Q_ASSERT(QFileInfo::exists(propagator()->fullLocalPath(_item->_originalFile)) || QFileInfo::exists(existingFile));
     if (_item->_file != _item->_renameTarget) {
         propagator()->reportProgress(*_item, 0);
         qCDebug(lcPropagateLocalRename) << "MOVE " << existingFile << " => " << targetFile;
@@ -246,7 +302,17 @@ void PropagateLocalRename::start()
         if (QString::compare(_item->_file, _item->_renameTarget, Qt::CaseInsensitive) != 0
             && propagator()->localFileNameClash(_item->_renameTarget)) {
 
-            qCInfo(lcPropagateLocalRename) << "renaming a case clashed file" << _item->_file << _item->_renameTarget;
+            qCInfo(lcPropagateLocalRename) << "renaming a case clashed item" << _item->_file << _item->_renameTarget;
+            if (_item->isDirectory()) {
+                // #HotFix
+                // fix a crash (we can not create a conflicted copy for folders)
+                // right now, the conflict resolution will not even work for this scenario with folders,
+                // but, let's fix it step by step, this will be a second stage
+                done(SyncFileItem::FileNameClash,
+                     tr("Folder %1 cannot be renamed because of a local file or folder name clash!").arg(_item->_file),
+                     ErrorCategory::GenericError);
+                return;
+            }
             const auto caseClashConflictResult = propagator()->createCaseClashConflict(_item, existingFile);
             if (caseClashConflictResult) {
                 done(SyncFileItem::SoftError, *caseClashConflictResult, ErrorCategory::GenericError);
@@ -256,12 +322,76 @@ void PropagateLocalRename::start()
             return;
         }
 
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+        auto targetParentFolderPath = std::filesystem::path{};
+        auto targetParentFolderWasReadOnly = false;
+        try {
+            const auto newDirPath = std::filesystem::path{targetFile.toStdWString()};
+            Q_ASSERT(newDirPath.has_parent_path());
+            targetParentFolderPath = newDirPath.parent_path();
+            if (FileSystem::isFolderReadOnly(targetParentFolderPath)) {
+                targetParentFolderWasReadOnly = true;
+                FileSystem::setFolderPermissions(QString::fromStdWString(targetParentFolderPath.wstring()), FileSystem::FolderPermissions::ReadWrite);
+                emit propagator()->touchedFile(QString::fromStdWString(targetParentFolderPath.wstring()));
+            }
+        }
+        catch (const std::filesystem::filesystem_error &e)
+        {
+            qCWarning(lcPropagateLocalRename) << "exception when checking parent folder access rights" << e.what() << e.path1().c_str() << e.path2().c_str();
+        }
+
+        auto originParentFolderPath = std::filesystem::path{};
+        auto originParentFolderWasReadOnly = false;
+        try {
+            const auto newDirPath = std::filesystem::path{existingFile.toStdWString()};
+            Q_ASSERT(newDirPath.has_parent_path());
+            originParentFolderPath = newDirPath.parent_path();
+            if (FileSystem::isFolderReadOnly(originParentFolderPath)) {
+                originParentFolderWasReadOnly = true;
+                FileSystem::setFolderPermissions(QString::fromStdWString(originParentFolderPath.wstring()), FileSystem::FolderPermissions::ReadWrite);
+                emit propagator()->touchedFile(QString::fromStdWString(originParentFolderPath.wstring()));
+            }
+        }
+        catch (const std::filesystem::filesystem_error &e)
+        {
+            qCWarning(lcPropagateLocalRename) << "exception when checking parent folder access rights" << e.what() << e.path1().c_str() << e.path2().c_str();
+        }
+
+        const auto restoreTargetPermissions = [this] (const auto &parentFolderPath) {
+            try {
+                FileSystem::setFolderPermissions(QString::fromStdWString(parentFolderPath.wstring()), FileSystem::FolderPermissions::ReadOnly);
+                emit propagator()->touchedFile(QString::fromStdWString(parentFolderPath.wstring()));
+            }
+            catch (const std::filesystem::filesystem_error &e)
+            {
+                qCWarning(lcPropagateLocalRename) << "exception when checking parent folder access rights" << e.what() << e.path1().c_str() << e.path2().c_str();
+            }
+        };
+#endif
+
         emit propagator()->touchedFile(existingFile);
         emit propagator()->touchedFile(targetFile);
         if (QString renameError; !FileSystem::rename(existingFile, targetFile, &renameError)) {
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+            if (targetParentFolderWasReadOnly) {
+                restoreTargetPermissions(targetParentFolderPath);
+            }
+            if (originParentFolderWasReadOnly) {
+                restoreTargetPermissions(originParentFolderPath);
+            }
+#endif
             done(SyncFileItem::NormalError, renameError, ErrorCategory::GenericError);
             return;
         }
+
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+        if (targetParentFolderWasReadOnly) {
+            restoreTargetPermissions(targetParentFolderPath);
+        }
+        if (originParentFolderWasReadOnly) {
+            restoreTargetPermissions(originParentFolderPath);
+        }
+#endif
     }
 
     SyncJournalFileRecord oldRecord;
