@@ -17,8 +17,9 @@
 #include <QJsonValue>
 
 #include <memory>
-
-
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+#include <filesystem>
+#endif
 
 PathComponents::PathComponents(const char *path)
     : PathComponents { QString::fromUtf8(path) }
@@ -48,10 +49,15 @@ PathComponents PathComponents::subComponents() const &
 void DiskFileModifier::remove(const QString &relativePath)
 {
     QFileInfo fi { _rootDir.filePath(relativePath) };
-    if (fi.isFile())
+    if (fi.isFile()) {
         QVERIFY(_rootDir.remove(relativePath));
-    else
-        QVERIFY(QDir { fi.filePath() }.removeRecursively());
+    } else {
+#if !defined(Q_OS_MACOS) || __MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_15
+        const auto pathToDelete = fi.filePath().toStdWString();
+        std::filesystem::permissions(pathToDelete, std::filesystem::perms::owner_exec, std::filesystem::perm_options::add);
+        QVERIFY(std::filesystem::remove_all(pathToDelete));
+#endif
+    }
 }
 
 void DiskFileModifier::insert(const QString &relativePath, qint64 size, char contentChar)
@@ -252,6 +258,21 @@ FileInfo *FileInfo::find(PathComponents pathComponents, const bool invalidateEta
     return nullptr;
 }
 
+FileInfo FileInfo::findRecursive(PathComponents pathComponents, const bool invalidateEtags)
+{
+    auto result = find({pathComponents.takeFirst()}, invalidateEtags);
+    if (!result) {
+        return *result;
+    }
+    for (const auto &pathComponent : pathComponents) {
+        if (!result) {
+            break;
+        }
+        result = result->find({pathComponent});
+    }
+    return *result;
+}
+
 FileInfo *FileInfo::createDir(const QString &relativePath)
 {
     const PathComponents pathComponents { relativePath };
